/*
Software to convert a line-CCD plus a bunch of LEDs to a touchscreen.
(C) 2011 Jeroen Domburg (jeroen AT spritesmods.com)

This program is free software: you can redistribute it and/or modify
t under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
	    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
			    
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include "serial.h"
#include <termios.h>

//See main.c of the avr software for a short protocol description.

#define BAUDRATE B115200
#define _POSIX_SOURCE 1 /* POSIX compliant source */

int serialOpen(char *port) {
    struct termios oldtio,newtio;
	int com=open(port, O_RDONLY);

    com = open("/dev/ttyUSB0", O_RDWR | O_NOCTTY );
    if (com <0) {perror("Opening comport"); exit(-1); }
    tcgetattr(com,&oldtio); // save current port settings
    bzero(&newtio, sizeof(newtio));
    newtio.c_cflag =  (CS8 | CREAD | CLOCAL);
    newtio.c_lflag = 0;
    newtio.c_iflag = IGNPAR;
    newtio.c_oflag = 0;
    cfsetospeed(&newtio,BAUDRATE);
    
    newtio.c_cc[VTIME]    = 0;   // inter-character timer unused 
    newtio.c_cc[VMIN]     = 1;   // blocking read until 1 chars received 
      
    tcflush(com,TCIFLUSH);
    tcsetattr(com,TCSANOW,&newtio);

	return com;
}

static unsigned int get16b(int fd) {
	unsigned char buff[2];
	int i=0;
	while (i!=2){
		i+=read(fd, buff+i, 2-i);
	}
	return (buff[0]<<8)|(buff[1]);
}

int serialRead(int fd, BlackMark *meas, int max) {
	int x,n,i;
	n=0;

	x=0;
	while ((i=get16b(fd))!=0xAAFE) { 
		printf("Serial desynced: Got %04x waiting for 0xAAFE\n",i);
		//Read one extra to re-sync better.
		x++;
		if ((x&15)==0) read(fd, &i, 1);
	}

	x=1;
	while ((i=get16b(fd))!=0xAAFF) {
//		printf("Got %04x\n",i);
		if ((i&0xFF00)==0xAA00) {
			x=i&0xff;
		} else {
			meas[n].led=x-1;
			meas[n].start=i;
			meas[n].len=get16b(fd);
			if (meas[n].len<800) n++;
			if (n==max) return n;
		}
	}
//	printf("Got %04x, finishing with %i measurements\n",i,n);
	return n;
}
