/*
Software to convert a line-CCD plus a bunch of LEDs to a touchscreen.
(C) 2011 Jeroen Domburg (jeroen AT spritesmods.com)

This program is free software: you can redistribute it and/or modify
t under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
	    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
			    
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "vector.h"
#include <math.h>

//Resize a vector to a given length
void resizeVect(VectorStruct *v, double newlen) {
	double div=sqrt(v->x*v->x + v->y*v->y)/newlen;
	v->x/=div;
	v->y/=div;
}

static double dotproduct(VectorStruct p1, VectorStruct p2) {
	return p1.x*p2.x+p1.y*p2.y;
}

//Calculate the intersecting point of two lines. Source:
// http://www.cracktheinterview.net/adobe/322-line-segment-intersection.html
//(because my math is too rusty to trust the solution I worked out myself)
int getIntersectPoint(VectorStruct base1, VectorStruct dir1, VectorStruct base2, VectorStruct dir2, VectorStruct *res) {
	VectorStruct p;
	VectorStruct aminc;
	double h;
	resizeVect(&dir1, 1);
	resizeVect(&dir2, 1);
	p.x=-dir1.y; p.y=dir1.x;
	aminc.x=base1.x-base2.x;
	aminc.y=base1.y-base2.y;
	if (fabs(dotproduct(dir2, p))<.00001) return 0; //probably parallel
	h=dotproduct(aminc, p)/dotproduct(dir2, p);
	res->x=base2.x+dir2.x*h;
	res->y=base2.y+dir2.y*h;
	return 1;
}

